#!/bin/bash

export TAG="0.10"
export RUNTIME_IMAGE="registry.uipath.com/uipath/sf-k8-utils-rhel:$TAG"

info() {
  echo "[INFO] : $*"
}

warn() {
  echo "[WARN] : $*" >&2
}

error() {
  echo -e "\e[0;31m[ERROR] :\e[0m $*" >&2
  exit 1
}
error_without_exit() {
  echo -e "\e[0;31m[ERROR] :\e[0m $*" >&2
}

function set_runtime_image_offline() {
  # By default, We will look for uipath/sf-k8-utils-rhel image
  # If installer is old then only sf-k8-utils-rhel will be used
  image="uipath/sf-k8-utils-rhel"

  [[ -z "$(kubectl -n docker-registry get pod -l=app=docker-registry -o name)" ]] && return

  tag=$(find_latest_tag "${image}")
  if [ -z "${tag}" ]; then
    # Fallaback to check in case of old installer
    tag=$(find_latest_tag "sf-k8-utils-rhel")
    [[ ! -z "${tag}" ]] && image="sf-k8-utils-rhel"
  fi

  [[ ! -z "${tag}" ]] && TAG=$tag

  export RUNTIME_IMAGE="localhost:30071/$image:${TAG}"
}

#find_latest_tag will return the latest tag for given image available in cluster docker-registry
function find_latest_tag() {
	local image=$1
	local username password

	if kubectl get secret docker-registry-credentials -n uipath >/dev/null 2>&1; then
          password=$(kubectl get secrets/docker-registry-credentials -n "uipath" --template \{\{.data.password\}\} | base64 -d)
          username=$(kubectl get secrets/docker-registry-credentials -n "uipath" --template \{\{.data.username\}\} | base64 -d)
	fi

	cred="-u $username:$password"
	ret=$(curl -s -k $cred -w "%{http_code}" "https://localhost:30071/v2/$image/tags/list" -o /dev/null) || true
	if [[ "$ret" != "200" ]] && [[ "$ret" != "404" ]]; then
		warn "Not able to identify $image image tag. Registry is not accessible."
		return
	elif [[ "$ret" == "404" ]]; then
		warn "Image $image doesn't exist in registry."
		return
	fi

	origTag=""
	origDate=0
	for tag in $(curl -s -k $cred "https://localhost:30071/v2/$image/tags/list" | jq -r '.tags[]'); do
		date=$(date +%s -d $(curl -s -k $cred "https://localhost:30071/v2/$image/manifests/$tag"   | jq -r '.history[].v1Compatibility' | jq '.created'  |sort | tail -1 | tr -d '"'))
		if [[ $date -gt $origDate ]]; then
			origDate=$date
			origTag=$tag
		fi
	done

	echo $origTag
}

# fetch_hostname return hostname as it is.
# For k8s operation, use fetch_hostname_lowercase to get node name
function fetch_hostname() {
  hostname
}

# For k8s operation, node name should be in lowercase.
function fetch_hostname_lowercase() {
  local node_name
  node_name=$(xargs -0 < "/proc/$(pgrep kubelet)/cmdline" | tr ' ' '\n' | grep '\-\-hostname\-override' | cut -d'=' -f2)
  [[ -z "${node_name}" ]] && node_name=$(fetch_hostname | awk '{print tolower($0)}')
  echo "${node_name}"
}
